@@ -182,12 +182,28 @@ sub process {
 	}
     }
 
-    my $channel = &channel();
 
     if ($addressed) {
 	&status("$who is addressing me");
 	$lastaddressedby = $who;
 	$lastaddressedtime = time();
+
+	if ($message =~ /^showmode/i ) {
+	    if ($msgType =~ /public/) {
+		if (($param{'addressing'} eq 'REQUIRE') && !$addressed) {
+		    return "NOREPLY";
+		} else {
+		    &performSay ($who.", addressing me is currently in $param{addressing} mode.");
+		    return "NOREPLY";
+		}
+	    } else {
+		&msg($who, "addressing me is currently in $param{addressing} mode");
+		return "NOREPLY";
+	    }
+	}
+	
+	my $channel = &channel();
+
     } else {
 	my ($now, $diff);
 	$now = time();
@@ -305,6 +321,50 @@ sub process {
 	}
     }
 
+
+    # Aldebaran++ !
+    if ($param{"shutup"} and $message =~ /^\s*wake\s*up\s*$/i ) {
+	if ($msgType =~ /public/) {
+	    if ($addressed) {
+		if (rand() > 0.5) {
+		    &performSay("Ok, ".$who.", I'll start talking.");
+		    &status("Changing to Optional mode");
+		    $param{'addressing'} = 'OPTIONAL';
+		    return "NOREPLY";
+		} else {
+		    &performSay(":O");
+		    return "NOREPLY";
+		}
+	    }
+	} else {
+	    &msg($who, "OK, I'll start talking.");
+	    $param{'addressing'} = 'OPTIONAL';
+	    &status("Changing to Optional mode");
+	    return "NOREPLY";
+	}
+    }
+ 
+    if ($param{"shutup"} and $message =~ /^\s*shut\s*up\s*$/i ) {
+	if ($msgType =~ /public/) {
+	    if ($addressed) {
+		if (rand() > 0.5) {
+		    &performSay("Sorry, ".$who.", I'll keep my mouth shut. ");
+		    $param{'addressing'} = 'REQUIRE';
+		    &status("Changing to Require mode");
+		    return "NOREPLY";
+		} else {
+		    &performSay(":X");
+		    return "NOREPLY";
+		}
+	    } 
+	} else {
+	    &msg($who, "Sorry, I'll try to be quiet.");
+	    $param{'addressing'} = 'REQUIRE';
+	    &status("Changing to Require mode");
+	    return "NOREPLY";
+	}
+    }
+
     $target = $who;
 
     $skipReply = 0;
@@ -329,20 +389,50 @@ sub process {
 	}
     }
 
+    # here's where the external routines get called.
+    # if they return anything but null, that's the "answer".
+    my $mr = &myRoutines();
+
+    if ($mr =~ /\S/) {
+	&status("myRoutines: $mr");
+	return $mr;
+    }
+
     # might want to take this out.
 
-    if ($addressed and $message =~ /^seen (\S+)/) {
+    if ($message =~ /^seen (\S+)/) {
 	my $person = $1;
 	$person =~ s/\?*\s*$//;
 	if ($seen{lc $person}) {
 	    my ($when,$what) = split /$;/, $seen{lc $person};
+	    my $howlong = time() - $when;
 	    $when = localtime $when;
-#	    $when = localtime($seen{lc $person});
+
+	    my $tstring = ($howlong % 60). " seconds ago";
+	    $howlong = int($howlong / 60);
+	    
+	    if ($howlong % 60) {
+		$tstring = ($howlong % 60). " minutes and $tstring";
+	    }
+	    $howlong = int($howlong / 60);
+
+	    if ($howlong % 24) {
+		$tstring = ($howlong % 24). " hours, $tstring";
+	    }
+	    $howlong = int($howlong / 24);
+
+	    if ($howlong % 365) {
+		$tstring = ($howlong % 365). " days, $tstring";
+	    }
+	    $howlong = int($howlong / 365);
+	    if ($howlong > 0) {
+		$tstring = "$howlong years, $tstring";
+	    }
 	    
 	    if ($msgType =~ /public/) {
-		&performSay("$person was last seen on IRC $when, saying: $what");
+		&performSay("$person was last seen on IRC $tstring, saying: $what [$when]");
 	    } else {
-		&msg($who, "$person was last seen on IRC $when, saying: $what");
+		&msg($who, "$person was last seen on IRC $tstring, saying: $what [$when]");
 	    }
 	    return 'NOREPLY';
 	}
@@ -355,32 +445,11 @@ sub process {
 	return 'NOREPLY';
     }
 
-    if (0 and $message =~ /^timer\s+(\d+)$/i) {
-	status("$who set a timer for $1 seconds");
-	alarm $1;
-	return '';
-    }
-
-    # here's where the external routines get called.
-    # if they return anything but null, that's the "answer".
-    my $mr = &myRoutines();
-
-    if ($mr =~ /\S/) {
-	&status("myRoutines: $mr");
-	return $mr;
-    }
-
     if ($message =~ /^\s*heya?,? /) {
 	return unless $addressed;
 	# greetings
     }
 
-    if ($message =~ /^\s*h(h|e|a|\s+)+\s*$/) { # laughter
-	return "";
-    }
-
-
-  
     # Gotta be gender-neutral here... we're sensitive to purl's needs. :-)
     if ($message =~ /(good( fuckin['g]?)? (bo(t|y)|g([ui]|r+)rl))|(bot( |\-)?snack)/i) {
 	&status("random praise");
@@ -512,15 +581,17 @@ sub process {
     if ($msgType !~ /public/) {
 	# accept only msgs leaves/joins
 	my($ok_to_join);
-	if ($message =~ /join ((\&|\#)\S+)/i) {
-	    my($which) = $1;
+        if ($message =~ /join ([&#]\S+)(?:\s+(\S+))?/i) {
+            # Thanks to Eden Li (tile) for the channel key patch
+            my($which, $key) = ($1, $2);
+            $key = defined ($key) ? " $key" : "";
 	    foreach $chan (split(/\s+/, $param{'allowed_channels'})) {
 		if (lc($which) eq lc($chan)) {
-		    $ok_to_join = $which;
+		    $ok_to_join = $which . $key;
 		    last;
 		}
 	    }
-	    if (IsFlag("o")) { $ok_to_join = $which };
+	    if (IsFlag("o")) { $ok_to_join = $which.$key };
 	    if ($ok_to_join) {
 		if (IsFlag("c") ne "c") {
 		    &msg($who, "You don't have the channel flag");
