@@ -97,7 +97,7 @@ sub doQuestion {
     if (($addressed) && ($in =~ /^statu?s/)) {
 	$upString = &timeToString(time()-$startTime);
 	$eTime = &get("is", "the qEpochDate");
-	return "Since $setup_time, there have been $updateCount modifications and $questionCount questions.  I have been awake for ".$upString." this session, and currently reference ".$factoidCount." factoids.";
+	return "Since $setup_time, there have been $updateCount modifications and $questionCount questions.  I have been awake for $upString this session, and currently reference $factoidCount factoids. Addressing is in ".lc($param{addressing})." mode.";
 	#  Since ".$is{"the qEpochDate"}." there have been about ".$is{"the qCount"}." questions total.";
     }
 
